@@ -1,5 +1,5 @@
 ;; shepherd.scm -- The daemon shepherd.
-;; Copyright (C) 2013, 2014, 2016, 2018, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013-2014, 2016, 2018-2020, 2022 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2018 Carlo Zancanaro <carlo@zancanaro.id.au>
 ;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
@@ -20,10 +20,11 @@
 ;; along with the GNU Shepherd.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (shepherd)
+  #:use-module ((fibers)
+                #:hide (sleep))                   ;avoid Guile warning
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module (ice-9 rdelim)   ;; Line-based I/O.
-  #:autoload   (ice-9 readline) (activate-readline) ;for interactive use
   #:use-module ((ice-9 threads) #:select (all-threads))
   #:use-module (oop goops)      ;; Defining classes and methods.
   #:use-module (srfi srfi-1)    ;; List library.
@@ -38,6 +39,7 @@
   #:export (main))
 
 
+(define sleep (@ (fibers) sleep))
 
 (define (open-server-socket file-name)
   "Open a socket at FILE-NAME, and listen for connections there."
@@ -50,23 +52,51 @@
       (listen sock 10)
       sock)))
 
+(define-syntax-rule (unwind-protect body ... conclude)
+  "Evaluate BODY... and return its result(s), but always evaluate CONCLUDE
+before leaving, even if an exception is raised.
+
+This is *not* implemented with 'dynamic-wind' in order to play well with
+delimited continuations and fibers."
+  (let ((conclusion (lambda () conclude)))
+    (catch #t
+      (lambda ()
+        (call-with-values
+            (lambda ()
+              body ...)
+          (lambda results
+            (conclusion)
+            (apply values results))))
+      (lambda args
+        (conclusion)
+        (apply throw args)))))
+
 (define (call-with-server-socket file-name proc)
   "Call PROC, passing it a listening socket at FILE-NAME and deleting the
 socket file at FILE-NAME upon exit of PROC.  Return the values of PROC."
-  (let ((sock (open-server-socket file-name)))
-    (dynamic-wind
-      noop
-      (lambda () (proc sock))
-      (lambda ()
-        (close sock)
-        (catch-system-error (delete-file file-name))))))
+  (let ((sock (catch 'system-error
+                (lambda ()
+                  (open-server-socket file-name))
+                (lambda args
+                  (match args
+                    ((key proc . _)
+                     (report-error (l10n "while opening socket '~a': ~a: ~a~%")
+                                   file-name proc
+                                   (strerror (system-error-errno args)))
+                     ;; Stop services that were started from the config file
+                     ;; and quit.
+                     (stop 'root)))))))
+    (unwind-protect (proc sock)
+                    (begin
+                      (close sock)
+                      (catch-system-error (delete-file file-name))))))
 
 (define (maybe-signal-port signals)
   "Return a signal port for SIGNALS, using 'signalfd' on GNU/Linux, or #f if
 that is not supported."
   (catch 'system-error
     (lambda ()
-      (let ((port (signalfd -1 signals)))
+      (let ((port (non-blocking-port (signalfd -1 signals))))
         ;; As per the signalfd(2) man page, block SIGNALS.  The tricky bit is
         ;; that SIGNALS must be blocked for all the threads; new threads will
         ;; inherit the signal mask, but we must ensure that neither Guile's
@@ -107,7 +137,11 @@ already ~a threads running, disabling 'signalfd' support")
         ((= signal SIGINT)
          (lambda _ (handle-SIGINT)))
         ((memv signal (list SIGTERM SIGHUP))
-         (lambda _ (stop root-service)))
+         (lambda _
+           (catch 'quit
+             (lambda ()
+               (stop root-service))
+             quit-exception-handler)))
         (else
          (const #f))))
 
@@ -116,6 +150,79 @@ already ~a threads running, disabling 'signalfd' support")
   (let ((signal (consume-signalfd-siginfo port)))
     ((signal-handler signal))))
 
+
+(define* (run-daemon #:key (config-file (default-config-file)) persistency
+                     socket-file pid-file signal-port poll-services?)
+  ;; This _must_ succeed.  (We could also put the `catch' around
+  ;; `main', but it is often useful to get the backtrace, and
+  ;; `caught-error' does not do this yet.)
+  (catch #t
+    (lambda ()
+      (load-in-user-module (or config-file (default-config-file))))
+    (lambda (key . args)
+      (caught-error key args)
+      (quit 1)))
+  ;; Start what was started last time.
+  (and persistency
+       (catch 'system-error
+         (lambda ()
+           (start-in-order (read (open-input-file
+                                  persistency-state-file))))
+         (lambda (key . args)
+           (apply format #f (gettext (cadr args)) (caddr args))
+           (quit 1))))
+
+  ;; Ignore SIGPIPE so that we don't die if a client closes the connection
+  ;; prematurely.
+  (sigaction SIGPIPE SIG_IGN)
+
+  (if (not socket-file)
+      ;; Get commands from the standard input port.
+      (process-textual-commands (current-input-port))
+      ;; Process the data arriving at a socket.
+      (call-with-server-socket
+       socket-file
+       (lambda (sock)
+
+         ;; Possibly write out our PID, which means we're ready to accept
+         ;; connections.  XXX: What if we daemonized already?
+         (match pid-file
+           ((? string? file)
+            (with-atomic-file-output pid-file
+              (cute display (getpid) <>)))
+           (#t (display (getpid)))
+           (_  #t))
+
+         ;; Spawn a signal handling fiber.
+         (spawn-fiber
+          (if signal-port
+              (lambda ()
+                (let loop ()
+                  (handle-signal-port signal-port)
+                  (loop)))
+              (lambda ()
+                ;; When not using signalfd(2), there's always a time window
+                ;; before 'select' during which a handler async can be
+                ;; queued but not executed.  Work around it by exiting
+                ;; 'select' every few seconds.
+                (let loop ()
+                  (sleep (if poll-services? 0.5 30))
+                  (when poll-services?
+                    (check-for-dead-services))
+                  (loop)))))
+
+         ;; Enter some sort of a REPL for commands.
+         (let next-command ()
+           (match (accept sock)
+             ((command-source . client-address)
+              (setvbuf command-source (buffering-mode block) 1024)
+              (spawn-fiber
+               (lambda ()
+                 (process-connection command-source))))
+             (_ #f))
+
+           (next-command))))))
+
 
 ;; Main program.
 (define (main . args)
@@ -217,20 +324,8 @@ already ~a threads running, disabling 'signalfd' support")
 		    (l10n "get commands from socket FILE or from stdin (-)")
 		    #:action (lambda (file)
 			       (set! socket-file
-				     (if (not (string=? file "-"))
-					 file
-				       ;; We will read commands
-				       ;; from stdin, thus we
-				       ;; enable readline if it is
-				       ;; a non-dumb terminal.
-				       (and (isatty? (current-input-port))
-					    (not (string=? (getenv "TERM")
-							   "dumb"))
-					    (begin
-					      (activate-readline)
-					      ;; Finally, indicate that
-					      ;; we use no socket.
-					      #f)))))))
+				     (and (not (string=? file "-"))
+					  file)))))
     ;; We do this early so that we can abort early if necessary.
     (and socket-file
          (verify-dir (dirname socket-file) #:secure? secure))
@@ -257,6 +352,8 @@ already ~a threads running, disabling 'signalfd' support")
                          (%make-void-port "w")
                          (current-output-port)))))
 
+      (set-port-encoding! (log-output-port) "UTF-8")
+
       ;; Start the 'root' service.
       (start root-service)
 
@@ -286,82 +383,25 @@ already ~a threads running, disabling 'signalfd' support")
                   (sigaction signal (signal-handler signal)))
                 (delete SIGCHLD %precious-signals))
 
-      ;; This _must_ succeed.  (We could also put the `catch' around
-      ;; `main', but it is often useful to get the backtrace, and
-      ;; `caught-error' does not do this yet.)
-      (catch #t
-        (lambda ()
-          (load-in-user-module (or config-file (default-config-file))))
-        (lambda (key . args)
-          (caught-error key args)
-          (quit 1)))
-      ;; Start what was started last time.
-      (and persistency
-           (catch 'system-error
-             (lambda ()
-               (start-in-order (read (open-input-file
-                                      persistency-state-file))))
-             (lambda (key . args)
-               (apply format #f (gettext (cadr args)) (caddr args))
-               (quit 1))))
-
-      ;; Ignore SIGPIPE so that we don't die if a client closes the connection
-      ;; prematurely.
-      (sigaction SIGPIPE SIG_IGN)
-
-      (if (not socket-file)
-          ;; Get commands from the standard input port.
-          (process-textual-commands (current-input-port))
-          ;; Process the data arriving at a socket.
-          (call-with-server-socket
-           socket-file
-           (lambda (sock)
-
-             ;; Possibly write out our PID, which means we're ready to accept
-             ;; connections.  XXX: What if we daemonized already?
-             (match pid-file
-               ((? string? file)
-                (with-atomic-file-output pid-file
-                  (cute display (getpid) <>)))
-               (#t (display (getpid)))
-               (_  #t))
-
-             ;; XXX: This call mostly to resolve 'handle-SIGCHLD' upfront.
-             ;; This works around Guile 3.0.2 occasionally failing with:
-             ;; "Failed to autoload handle-SIGCHLD in (ice-9 readline):"
-             (handle-SIGCHLD)
-
-             (let next-command ((ports (if signal-port
-                                           (list signal-port sock)
-                                           (list sock))))
-               (define (read-from sock)
-                 (match (accept sock)
-                   ((command-source . client-address)
-                    (setvbuf command-source (buffering-mode block) 1024)
-                    (process-connection command-source))
-                   (_ #f)))
-
-               ;; When not using signalfd(2), there's always a time window
-               ;; before 'select' during which a handler async can be queued
-               ;; but not executed.  Work around it by exiting 'select' every
-               ;; few seconds.
-               (match (select ports (list) (list)
-                              (and (not signal-port)
-                                   (if poll-services? 0.5 30)))
-                 (((port _ ...) _ _)
-                  (if (and signal-port (eq? port signal-port))
-                      (handle-signal-port port)
-                      (read-from sock)))
-                 (_
-                  ;; 'select' returned an empty set, probably due to EINTR.
-                  ;; Explicitly call the SIGCHLD handler because we cannot be
-                  ;; sure the async will be queued and executed before we call
-                  ;; 'select' again.
-                  (handle-SIGCHLD)))
-
-               (when poll-services?
-                 (check-for-dead-services))
-               (next-command ports))))))))
+      ;; Run Fibers in such a way that it does not create any POSIX thread,
+      ;; because POSIX threads and 'fork' cannot be used together.
+      (run-fibers
+       (lambda ()
+         (catch 'quit
+           (lambda ()
+             (run-daemon #:socket-file socket-file
+                         #:config-file config-file
+                         #:pid-file pid-file
+                         #:signal-port signal-port
+                         #:poll-services? poll-services?
+                         #:persistency persistency))
+           (case-lambda
+             ((key value . _)
+              (primitive-exit value))
+             ((key)
+              (primitive-exit 0)))))
+       #:parallelism 1  ;don't create POSIX threads
+       #:hz 0))))       ;disable preemption, which would require POSIX threads
 
 ;; Start all of SERVICES, which is a list of canonical names (FIXME?),
 ;; but in a order where all dependencies are fulfilled before we
@@ -406,17 +446,20 @@ already ~a threads running, disabling 'signalfd' support")
 
   ;; Most likely we're receiving 'quit' from the 'stop' method of
   ;; ROOT-SERVICE.  So, if we're running as 'root', just reboot.
-  (if (zero? (getuid))
+  (if (and (zero? (getuid)) (= 1 (getpid)))
       (begin
         (local-output (l10n "Rebooting..."))
         (reboot))
-      (quit)))
+      (begin
+        (local-output (l10n "Exiting."))
+        (primitive-exit 0))))              ;leave without going through Fibers
 
 (define (process-command command port)
   "Interpret COMMAND, a command sent by the user, represented as a
 <shepherd-command> object.  Send the reply to PORT."
   (match command
-    (($ <shepherd-command> the-action service-symbol (args ...) dir)
+    (($ <shepherd-command> the-action service-symbol (args ...)
+                           directory)             ;ignored
 
      ;; We have to catch `quit' so that we can send the terminator
      ;; line to herd before we actually quit.
@@ -449,15 +492,14 @@ already ~a threads running, disabling 'signalfd' support")
                                    port)))
 
              (define result
-               (with-directory-excursion dir
-                 (case the-action
-                   ((start) (apply start service-symbol args))
-                   ((stop) (apply stop service-symbol args))
-                   ((enforce) (apply enforce service-symbol args))
+               (case the-action
+                 ((start) (apply start service-symbol args))
+                 ((stop) (apply stop service-symbol args))
+                 ((enforce) (apply enforce service-symbol args))
 
-                   ;; Actions which have the semantics of `action' are
-                   ;; handled there.
-                   (else (apply action service-symbol the-action args)))))
+                 ;; Actions which have the semantics of `action' are
+                 ;; handled there.
+                 (else (apply action service-symbol the-action args))))
 
              (write-reply (command-reply command result #f (get-messages))
                           port))))
@@ -480,7 +522,7 @@ would write them on the 'herd' command line."
              (process-command (shepherd-command (string->symbol action)
                                                 (string->symbol service)
                                                 #:arguments arguments)
-                              port))
+                              (current-output-port)))
             (_
              (local-output (l10n "invalid command line") line)))
           (loop (read-line port))))))
