@@ -1,5 +1,5 @@
 # GNU Shepherd --- Make sure shepherd tolerates misbehaved clients.
-# Copyright © 2016 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2016, 2018 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -65,10 +65,10 @@ $herd status			# still here?
 $herd status			# still here?
 
 "$GUILE" -c "
-(use-modules (shepherd comm))
+(use-modules (shepherd comm) (shepherd support))
 
 (let ((sock (open-connection \"$socket\")))
-  (setvbuf sock _IOFBF 5000)
+  (setvbuf sock (buffering-mode block) 5000)
   (write-command (shepherd-command 'status 'root) sock)
 
   ;; Close prematurely, right after sending the command.
